/// <reference types="serviceworker" />
import { FulfillList, StatusMessage, RequestMessage, ResponseMessage } from './Message';
/**
 * A fetch forwarder
 * - For captured requests, message back to client.
 * - For messaged requests, message back the real response.
 */
export default class Worker {
    scope: ServiceWorkerGlobalScope;
    portMap: Map<string, MessagePort>;
    fulfillListMap: Map<MessagePort, FulfillList>;
    private beActive;
    constructor(scope: ServiceWorkerGlobalScope);
    /**
     * For captured requests, message back to the client.
     */
    onFetch: (event: FetchEvent) => void;
    onMessage: (event: MessageEvent) => void;
    /**
     * For messaged requests, respond with real responses.
     */
    onRequestMessage: (event: MessageEvent<RequestMessage>) => Promise<void>;
    /**
     * For messaged responses, treat as a previous captured request's response.
     */
    onResponseMessage: (event: MessageEvent<ResponseMessage>) => Promise<void>;
    onStatusMessage: (event: MessageEvent<StatusMessage>) => Promise<void>;
    isActive(): boolean;
    /**
     * Start capturing requests and receiving messages.
     */
    activate(): void;
    /**
     * Stop capturing requests and receiving messages.
     */
    restore(): void;
}
//# sourceMappingURL=Worker.d.ts.map